==> pyCOMPS-2.3.1-py2.py3-none-any.whl: COMPS/AuthManager.py <==
import os
import sys
import errno
import logging
import platform
import getpass
import tempfile
if sys.version_info[0] == 2:
    import Tkinter as tk
else:
    import tkinter as tk
from datetime import datetime, timedelta
import COMPS

logger = logging.getLogger('COMPS.AuthManager')

class AuthManager(object):
    """
    Manage authentication to COMPS.
    """
    __comps_auth_token_key = 'X-COMPS-Token'
    __comps_client_version = 10
    __token_filename_format = 'COMPS_Authtoken_%s_%s.txt'
    __token_username_index = 2
    __token_expiration_index = 6
    __token_groups_index = 12
    __token_environments_index = 13
    __token_renewal_buffer = 120

    def __init__(self, hoststring, verify_certs=False):
        self._hoststring = os.environ.get('COMPS_SERVER') or hoststring
        self._hoststring = self._hoststring.rstrip('/')

        if self._hoststring.rstrip('/') != hoststring.rstrip('/'):
            logger.info('Overriding hoststring with COMPS_SERVER environment variable')

        self._verify_certs = verify_certs

        self._auth_token = None
        self._username = None
        self._group_list = None
        self._env_list = None
        self._token_expiration = datetime.min
        self._token_renewal_time = None

    @property
    def username(self):
        return self._username

    @property
    def hoststring(self):
        return self._hoststring

    @property
    def groups(self):
        return self._group_list

    @property
    def environments(self):
        return self._env_list

    def has_auth_token(self):
        return self._auth_token is not None

    def get_auth_token(self):
        now = datetime.utcnow()

        if (self._auth_token is None or      # <--- first time in this execution that we're getting an auth token
                now > self._token_renewal_time):  # <--- token is expired or near expiry, so check to see whether anyone else (e.g. another thread) has cached a newer token

            token_filename = self.__get_token_path()

            if os.path.exists(token_filename):
                with open(token_filename, 'r') as tf:
                    token_content = tf.readlines()
                    token = token_content[0] if token_content else None

                if token:
                    self.__process_token(token)

        if now > self._token_expiration:
            self.__acquire_credentials()
        elif now > self._token_renewal_time:
            self.__renew_auth_token()

        return self.__comps_auth_token_key, self._auth_token

    def clear_auth_token(self):
        path = self.__get_token_path()

        # delete file if it exists
        try:
            os.remove(path)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                logger.error('Error deleting cached credentials: {0}'.format(e.message))

        self._auth_token = None

    @staticmethod
    def get_environment_macros(environment_name):
        """
        Retrieve the environment macros for a COMPS environment.

        This may be a somewhat temporary requirement until the Asset Service handles
        file dependencies more completely (allows uploads, etc).

        :param environment_name: the COMPS environment to retrieve macros for
        :return: a dictionary of environment macro key/value pairs
        """
        path = '/Environments/{0}'.format(environment_name)
        resp = COMPS.Client.get(path)

        json_resp = resp.json()

        if 'Environments' not in json_resp or \
                len(json_resp['Environments']) != 1 or \
                'Macros' not in json_resp['Environments'][0]:
            logger.debug(json_resp)
            raise RuntimeError('Malformed Experiments retrieve response!')

        return json_resp['Environments'][0]['Macros']

    def __acquire_credentials(self):
        success = False

        logger.info('Logging into {0}'.format(self._hoststring))

        while not success:
            creds = self._prompt_user_for_creds()

            if creds is None:
                logger.info('User canceled login')
                break

            creds['ClientVersion'] = AuthManager.__comps_client_version

            logger.debug('Sending auth request')

            resp = COMPS.Client.post("/tokens"
                                     , include_comps_auth_token=False
                                     , http_err_handle_exceptions=[302, 401]
                                     , allow_redirects=False
                                     , json=creds
                                     , verify=self._verify_certs)

            creds = None

            if resp.status_code == 302:
                raise RuntimeError('Error contacting login endpoint; attempting to hit \'http\' protocol instead of \'https\' ?')
            elif resp.status_code == 401:
                logger.info('Bad username/password')
                continue

            if AuthManager.__comps_auth_token_key in resp.headers:
                token = resp.headers[AuthManager.__comps_auth_token_key]
                self.__process_token(token)
                self.__cache_token(token)

                success = True
            else:
                logger.error('Error attempting to validate user credentials')

    @classmethod
    def _prompt_user_for_creds(cls):
        master = tk.Tk()
        master.title('COMPS Login')
        master["padx"] = 12
        master["pady"] = 10

        input_frame = tk.Frame(master)
        input_frame.grid(row=0)

        username = tk.StringVar(master)
        password = tk.StringVar(master)
        ret_ok = tk.BooleanVar(master)

        tk.Label(input_frame, text='Username', padx=10).grid(row=0)
        usrbox = tk.Entry(input_frame, textvariable=username)
        usrbox.grid(row=0, column=1)
        usrbox.focus()

        tk.Label(input_frame, text='Password', padx=10, pady=5).grid(row=1)
        pwdbox = tk.Entry(input_frame, show='*', textvariable=password)
        pwdbox.grid(row=1, column=1)

        def inputchanged(*args):
            if username.get() and password.get():
                okay_button.config(state='normal')
            else:
                okay_button.config(state='disabled')

        def onok(evt=None):
            if okay_button.cget('state') == 'normal':
                ret_ok.set(True)
                master.destroy()

        def oncancel(evt=None):
            username.set('')
            password.set('')
            ret_ok.set(False)
            master.destroy()

        username.trace("w", inputchanged)
        password.trace("w", inputchanged)

        button_frame = tk.Frame(master)
        button_frame["pady"] = 4
        button_frame.grid(row=1)
        button_frame.columnconfigure(0, weight=1, pad=6)
        button_frame.columnconfigure(1, weight=1, pad=6)

        okay_button = tk.Button(button_frame, command=onok, text='OK', state="disabled", width=10)
        okay_button.grid(row=0, column=0)
        tk.Button(button_frame, command=oncancel, text='Cancel', width=10).grid(row=0, column=1)

        master.bind('<Return>', onok)
        master.bind('<Escape>', oncancel)

        # reposition to the middle of the screen
        master.update_idletasks()
        w = master.winfo_screenwidth()
        h = master.winfo_screenheight()
        size = tuple(int(_) for _ in master.geometry().split('+')[0].split('x'))
        x = w/2 - size[0]/2
        y = h/2 - size[1]/2
        master.geometry("%dx%d+%d+%d" % (size + (x, y)))

        # make sure the dialog is in the front
        if platform.system() is not 'Darwin':
            master.lift()
        else:
            # .lift() apparently doesn't work on OSX... try this instead
            os.system('''/usr/bin/osascript -e 'tell app "Finder" to set frontmost of process "Python" to true' ''')

        master.mainloop()

        if not ret_ok.get():
            raise RuntimeError('User canceled attempt to get security credentials.')

        if len(username.get()) == 0 or len(password.get()) == 0:
            return None
        else:
            return {'Username': username.get(), 'Password': password.get()}

    def __renew_auth_token(self):
        success = False

        while not success:
            resp = COMPS.Client.put("/tokens"
                                    , include_comps_auth_token=False    # Don't try to get another auth token, otherwise we get in an
                                                                        #  endless loop... we will manually include the old one below...
                                    , json={ 'ClientVersion': AuthManager.__comps_client_version}
                                    , headers={ AuthManager.__comps_auth_token_key: self._auth_token }
                                    , verify=self._verify_certs)

            if AuthManager.__comps_auth_token_key in resp.headers:
                token = resp.headers[AuthManager.__comps_auth_token_key]
                self.__process_token(token)
                self.__cache_token(token)

                success = True
            else:
                logger.error('Error attempting to renew user credentials')

    def __process_token(self, token):
        tokensplit = token.split(',', AuthManager.__token_environments_index + 2) # 0-based + an extra so we separate from anything afterwards
        
        try:
            self._token_expiration = datetime.strptime(tokensplit[AuthManager.__token_expiration_index], '%Y-%m-%d-%H-%M-%S')
            self._group_list = tuple(tokensplit[AuthManager.__token_groups_index].split('-'))
            self._env_list = tuple(tokensplit[AuthManager.__token_environments_index].split('-'))

            # for when parsing cached token
            if self._username is None:
                self._username = tokensplit[AuthManager.__token_username_index]
        except ValueError as e:  # todo: not ValueError
            # do stuff here
            pass
        
        self._token_renewal_time = self._token_expiration - timedelta(minutes=AuthManager.__token_renewal_buffer)
        self._auth_token = token

    def __cache_token(self, token):
        path = self.__get_token_path()

        logger.debug('Caching auth token to ' + path)
        try:
            with open(path, 'w') as tf:
                tf.write(token)
        except ValueError as e:
            logger.error("Failure caching auth-token: {0}".format(e.message))

    def __get_token_path(self):
        hoststring_repl = ''.join([c if c not in ':/' else '_' for c in self._hoststring])

        token_filename = os.path.join(tempfile.gettempdir(),
                                      AuthManager.__token_filename_format % (hoststring_repl, getpass.getuser()))

        return token_filename

==> pyCOMPS-2.3.3-py2.py3-none-any.whl: COMPS/AuthManager.py <==
import os
import sys
import errno
import logging
import platform
import getpass
import tempfile
if sys.version_info[0] == 2:
    import Tkinter as tk
else:
    import tkinter as tk
from datetime import datetime, timedelta
import COMPS

logger = logging.getLogger('COMPS.AuthManager')

class AuthManager(object):
    """
    Manage authentication to COMPS.
    """
    __comps_auth_token_key = 'X-COMPS-Token'
    __comps_client_version = 10
    __token_filename_format = 'COMPS_Authtoken_%s_%s.txt'
    __token_username_index = 2
    __token_expiration_index = 6
    __token_groups_index = 12
    __token_environments_index = 13
    __token_renewal_buffer = 120

    def __init__(self, hoststring, verify_certs=False):
        self._hoststring = os.environ.get('COMPS_SERVER') or hoststring
        self._hoststring = self._hoststring.rstrip('/')

        if self._hoststring.rstrip('/') != hoststring.rstrip('/'):
            logger.info('Overriding hoststring with COMPS_SERVER environment variable')

        self._verify_certs = verify_certs

        self._auth_token = None
        self._username = None
        self._group_list = None
        self._env_list = None
        self._token_expiration = datetime.min
        self._token_renewal_time = None

    @property
    def username(self):
        return self._username

    @property
    def hoststring(self):
        return self._hoststring

    @property
    def groups(self):
        return self._group_list

    @property
    def environments(self):
        return self._env_list

    def has_auth_token(self):
        return self._auth_token is not None

    def get_auth_token(self):
        now = datetime.utcnow()

        if (self._auth_token is None or      # <--- first time in this execution that we're getting an auth token
                now > self._token_renewal_time):  # <--- token is expired or near expiry, so check to see whether anyone else (e.g. another thread) has cached a newer token

            token_filename = self.__get_token_path()

            if os.path.exists(token_filename):
                with open(token_filename, 'r') as tf:
                    token_content = tf.readlines()
                    token = token_content[0] if token_content and token_content[0].strip() != '' else None

                if token:
                    self.__process_token(token)

        if now > self._token_expiration:
            self.__acquire_credentials()
        elif now > self._token_renewal_time:
            self.__renew_auth_token()

        return self.__comps_auth_token_key, self._auth_token

    def clear_auth_token(self):
        path = self.__get_token_path()

        # delete file if it exists
        try:
            os.remove(path)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                logger.error('Error deleting cached credentials: {0}'.format(e.message))

        self._auth_token = None

    @staticmethod
    def get_environment_macros(environment_name):
        """
        Retrieve the environment macros for a COMPS environment.

        This may be a somewhat temporary requirement until the Asset Service handles
        file dependencies more completely (allows uploads, etc).

        :param environment_name: the COMPS environment to retrieve macros for
        :return: a dictionary of environment macro key/value pairs
        """
        path = '/Environments/{0}'.format(environment_name)
        resp = COMPS.Client.get(path)

        json_resp = resp.json()

        if 'Environments' not in json_resp or \
                len(json_resp['Environments']) != 1 or \
                'Macros' not in json_resp['Environments'][0]:
            logger.debug(json_resp)
            raise RuntimeError('Malformed Experiments retrieve response!')

        return json_resp['Environments'][0]['Macros']

    def __acquire_credentials(self):
        success = False

        logger.info('Logging into {0}'.format(self._hoststring))

        while not success:
            creds = self._prompt_user_for_creds()

            if creds is None:
                logger.info('User canceled login')
                break

            creds['ClientVersion'] = AuthManager.__comps_client_version

            logger.debug('Sending auth request')

            resp = COMPS.Client.post("/tokens"
                                     , include_comps_auth_token=False
                                     , http_err_handle_exceptions=[302, 401]
                                     , allow_redirects=False
                                     , json=creds
                                     , verify=self._verify_certs)

            creds = None

            if resp.status_code == 302:
                raise RuntimeError('Error contacting login endpoint; attempting to hit \'http\' protocol instead of \'https\' ?')
            elif resp.status_code == 401:
                logger.info('Bad username/password')
                continue

            if AuthManager.__comps_auth_token_key in resp.headers:
                token = resp.headers[AuthManager.__comps_auth_token_key]
                self.__process_token(token)
                self.__cache_token(token)

                success = True
            else:
                logger.error('Error attempting to validate user credentials')

    @classmethod
    def _prompt_user_for_creds(cls):
        master = tk.Tk()
        master.title('COMPS Login')
        master["padx"] = 12
        master["pady"] = 10

        input_frame = tk.Frame(master)
        input_frame.grid(row=0)

        username = tk.StringVar(master)
        password = tk.StringVar(master)
        ret_ok = tk.BooleanVar(master)

        tk.Label(input_frame, text='Username', padx=10).grid(row=0)
        usrbox = tk.Entry(input_frame, textvariable=username)
        usrbox.grid(row=0, column=1)
        usrbox.focus()

        tk.Label(input_frame, text='Password', padx=10, pady=5).grid(row=1)
        pwdbox = tk.Entry(input_frame, show='*', textvariable=password)
        pwdbox.grid(row=1, column=1)

        def inputchanged(*args):
            if username.get() and password.get():
                okay_button.config(state='normal')
            else:
                okay_button.config(state='disabled')

        def onok(evt=None):
            if okay_button.cget('state') == 'normal':
                ret_ok.set(True)
                master.destroy()

        def oncancel(evt=None):
            username.set('')
            password.set('')
            ret_ok.set(False)
            master.destroy()

        username.trace("w", inputchanged)
        password.trace("w", inputchanged)

        button_frame = tk.Frame(master)
        button_frame["pady"] = 4
        button_frame.grid(row=1)
        button_frame.columnconfigure(0, weight=1, pad=6)
        button_frame.columnconfigure(1, weight=1, pad=6)

        okay_button = tk.Button(button_frame, command=onok, text='OK', state="disabled", width=10)
        okay_button.grid(row=0, column=0)
        tk.Button(button_frame, command=oncancel, text='Cancel', width=10).grid(row=0, column=1)

        master.bind('<Return>', onok)
        master.bind('<Escape>', oncancel)

        # reposition to the middle of the screen
        master.update_idletasks()
        w = master.winfo_screenwidth()
        h = master.winfo_screenheight()
        size = tuple(int(_) for _ in master.geometry().split('+')[0].split('x'))
        x = w/2 - size[0]/2
        y = h/2 - size[1]/2
        master.geometry("%dx%d+%d+%d" % (size + (x, y)))

        # make sure the dialog is in the front
        if platform.system() is not 'Darwin':
            master.lift()
        else:
            # .lift() apparently doesn't work on OSX... try this instead
            os.system('''/usr/bin/osascript -e 'tell app "Finder" to set frontmost of process "Python" to true' ''')

        master.mainloop()

        if not ret_ok.get():
            raise RuntimeError('User canceled attempt to get security credentials.')

        if len(username.get()) == 0 or len(password.get()) == 0:
            return None
        else:
            return {'Username': username.get(), 'Password': password.get()}

    def __renew_auth_token(self):
        success = False

        while not success:
            resp = COMPS.Client.put("/tokens"
                                    , include_comps_auth_token=False    # Don't try to get another auth token, otherwise we get in an
                                                                        #  endless loop... we will manually include the old one below...
                                    , json={ 'ClientVersion': AuthManager.__comps_client_version}
                                    , headers={ AuthManager.__comps_auth_token_key: self._auth_token }
                                    , verify=self._verify_certs)

            if AuthManager.__comps_auth_token_key in resp.headers:
                token = resp.headers[AuthManager.__comps_auth_token_key]
                self.__process_token(token)
                self.__cache_token(token)

                success = True
            else:
                logger.error('Error attempting to renew user credentials')

    def __process_token(self, token):
        tokensplit = token.split(',', AuthManager.__token_environments_index + 2) # 0-based + an extra so we separate from anything afterwards
        
        try:
            self._token_expiration = datetime.strptime(tokensplit[AuthManager.__token_expiration_index], '%Y-%m-%d-%H-%M-%S')
            self._group_list = tuple(tokensplit[AuthManager.__token_groups_index].split('-'))
            self._env_list = tuple(tokensplit[AuthManager.__token_environments_index].split('-'))

            # for when parsing cached token
            if self._username is None:
                self._username = tokensplit[AuthManager.__token_username_index]
        except (ValueError, IndexError) as e:
            raise RuntimeError('Invalid auth token: {}'.format(self.__get_token_path())) from None

        self._token_renewal_time = self._token_expiration - timedelta(minutes=AuthManager.__token_renewal_buffer)
        self._auth_token = token

    def __cache_token(self, token):
        path = self.__get_token_path()

        logger.debug('Caching auth token to ' + path)
        try:
            with open(path, 'w') as tf:
                tf.write(token)
        except ValueError as e:
            logger.error("Failure caching auth-token: {0}".format(e.message))

    def __get_token_path(self):
        hoststring_repl = ''.join([c if c not in ':/' else '_' for c in self._hoststring])

        token_filename = os.path.join(tempfile.gettempdir(),
                                      AuthManager.__token_filename_format % (hoststring_repl, getpass.getuser()))

        return token_filename
